import { ChangeDetectorRef, Component } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { MyPmstypeModel, PmstypeModel } from 'src/app/shared/model/pmstype.model';
import { FileService } from 'src/app/shared/services/file.service';
import { PmstypeService } from 'src/app/shared/services/pmstype.service';
@Component({
  selector: 'app-assessment-types',
  templateUrl: './assessment-types.component.html',
  styleUrls: ['./assessment-types.component.scss']
})
export class AssessmentTypesComponent {
  currentPage = 1
  page = Array.from({ length: 1 }, (_, i) => i + 1);
  search = ""
  numDataListChecked = 0
  isDataListChecked = false
  isDataListCheckedAll = false
  pmstype: { loading: boolean, select: PmstypeModel, dataList: { check: boolean, data: PmstypeModel }[] } = { loading: false, select: new MyPmstypeModel(), dataList: [] }
  modalStatus: "add" | "edit" | 'delete' | 'deleteGroup' = "add"

  selectedFile: File | null = null;
  selectedFileName: string = 'กรุณาเลือกไฟล์';
  constructor(private cdr: ChangeDetectorRef,
    private pmstypeService: PmstypeService,
    private toastr: ToastrService,
    private fileService: FileService
  ) {
  }

  ngOnInit(): void {
    this.getPmstypeList()
  }

  getPmstypeList() {
    this.pmstype.loading = true
    this.pmstypeService.getList().subscribe({
      next: response => {
        this.pmstype.dataList = response.map(x => ({ check: false, data: new MyPmstypeModel(x) }))
        this.isDataListCheckedAll = false
        this.dataListCheckAll()
        this.pmstype.loading = false
        this.searchChange()
        this.cdr.detectChanges()
      }, error: error => {
        this.pmstype.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  pmstypeListFilter() {
    return this.pmstype.dataList.filter(x => {
      const data = x.data
      const match = data.pmsTypeId.toLowerCase().includes(this.search.toLowerCase()) ||
        data.tdesc.toLowerCase().includes(this.search.toLowerCase()) ||
        data.shortName.toLowerCase().includes(this.search.toLowerCase())
      return match
    })
  }
  selectPmstype(data?: PmstypeModel) {
    this.pmstype.select = new MyPmstypeModel(data)
  }
  updatePmstype(typeApi: 'post' | 'delete') {
    let body: PmstypeModel | PmstypeModel[]
    switch (this.modalStatus) {
      case ('delete'): {
        body = [new MyPmstypeModel(this.pmstype.select)]
        break;
      }
      case ('deleteGroup'): {
        body = this.pmstype.dataList.filter(x => x.check).map(x => new MyPmstypeModel(x.data))
        break;
      }
      default: {
        body = new MyPmstypeModel(this.pmstype.select)
      }
    }
    this.pmstype.loading = true
    this.pmstypeService[typeApi]((body as any)).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getPmstypeList()
          this.searchChange()
        } else {
          this.showAlert(response.message, 'error')
          this.pmstype.loading = false
        }
        this.cdr.detectChanges()
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.pmstype.loading = false
        this.cdr.detectChanges()
      }
    })
  }

  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    })
  }


  onFileSelected(event: any) {
    this.selectedFile = event.target.files.length > 0 ? event.target.files[0] : null;
    this.selectedFileName = this.selectedFile?.name || "กรุณาเลือกไฟล์"
  }

  uploadFile() {
    if (!this.selectedFile) {
      alert('กรุณาเลือกไฟล์ก่อนอัปโหลด')
      return
    }
    const formData = new FormData();
    formData.append('file', this.selectedFile);
    this.pmstype.loading = true
    this.fileService.uploadExcel(formData, '').subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getPmstypeList()
        } else {
          this.showAlert(response.message, 'error')
          this.pmstype.loading = false
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.pmstype.loading = false
        this.cdr.detectChanges()
      }
    })
  }

  downloadFile() {
    const fileName = '.xlsx'
    this.fileService.downloadTemplate(fileName).subscribe({
      next: response => {
        const url = window.URL.createObjectURL(response);
        const a = document.createElement("a");
        a.href = url;
        a.download = fileName;
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        window.URL.revokeObjectURL(url);
      }, error: error => {
        this.showAlert(error.message, 'error')
      }
    })

  }


  dataListCheckAll() {
    const selectAll = this.isDataListCheckedAll;
    this.pmstype.dataList.filter(x => {
      const data = x.data
      const match = data.pmsTypeId.toLowerCase().includes(this.search.toLowerCase()) ||
        data.tdesc.toLowerCase().includes(this.search.toLowerCase()) ||
        data.shortName.toLowerCase().includes(this.search.toLowerCase())
      return match
    }).forEach(x => x.check = selectAll);
    this.dataListCheck();
  }

  dataListCheck() {
    const dataCheck = this.pmstypeListFilter();
    this.isDataListCheckedAll = dataCheck.length ? dataCheck.every(x => x.check) : false;
    this.numDataListChecked = this.pmstype.dataList.filter(x => x.check).length;
    this.isDataListChecked = Boolean(this.numDataListChecked)
  }
  clearPmstype(modalStatus: string) {
    if (modalStatus == 'add') {
      this.selectPmstype()
    } else if (modalStatus == 'edit') {
      this.selectPmstype(new MyPmstypeModel({ pmsTypeId: this.pmstype.select.pmsTypeId }))
    }
  }
  searchChange() {
    this.currentPage = 1
    this.page = Array.from({ length: Math.ceil(this.pmstypeListFilter().length / 10) }, (_, i) => i + 1);
    this.dataListCheck()
  }


}

